"""
This file contains the serializers (for the elsewhere defined data models)
"""

import logging
logger = logging.getLogger(__name__)

from rest_framework import serializers
from .. import models
from .specification import RelationalHyperlinkedModelSerializer


class SubtaskConnectorSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = models.SubtaskConnector
        fields = '__all__'


class SubtaskStateSerializer(serializers.ModelSerializer):
    class Meta:
        model = models.SubtaskState
        fields = '__all__'


class SubtaskTypeSerializer(serializers.ModelSerializer):
    class Meta:
        model = models.SubtaskType
        fields = '__all__'


class StationTypeSerializer(serializers.ModelSerializer):
    class Meta:
        model = models.StationType
        fields = '__all__'


class AlgorithmSerializer(serializers.ModelSerializer):
    class Meta:
        model = models.Algorithm
        fields = '__all__'


class ScheduleMethodSerializer(serializers.ModelSerializer):
    class Meta:
        model = models.ScheduleMethod
        fields = '__all__'


class SubtaskTemplateSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = models.SubtaskTemplate
        fields = '__all__'


class DefaultSubtaskTemplateSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = models.DefaultSubtaskTemplate
        fields = '__all__'


class DataproductSpecificationsTemplateSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = models.DataproductSpecificationsTemplate
        fields = '__all__'


class DefaultDataproductSpecificationsTemplateSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = models.DefaultDataproductSpecificationsTemplate
        fields = '__all__'


class SubtaskInputSelectionTemplateSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = models.SubtaskInputSelectionTemplate
        fields = '__all__'


class DataproductFeedbackTemplateSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = models.DataproductFeedbackTemplate
        fields = '__all__'


class SubtaskSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = models.Subtask
        fields = '__all__'


class SubtaskInputSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = models.SubtaskInput
        fields = '__all__'


class SubtaskOutputSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = models.SubtaskOutput
        fields = '__all__'


class DataproductSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = models.Dataproduct
        fields = '__all__'


class AntennaSetSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = models.AntennaSet
        fields = '__all__'


class DataproductTransformSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = models.DataproductTransform
        fields = '__all__'


class FilesystemSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = models.Filesystem
        fields = '__all__'


class ClusterSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = models.Cluster
        fields = '__all__'


class DataproductArchiveInfoSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = models.DataproductArchiveInfo
        fields = '__all__'


class DataproductHashSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = models.DataproductHash
        fields = '__all__'


class SubtaskSerializerJSONeditorOnline(RelationalHyperlinkedModelSerializer):

    # Create a JSON editor form to replace the simple text field based on the schema in the template that this
    # draft refers to. If that fails, the JSONField remains a standard text input.
    #
    # Note: I feel a bit uneasy with this since I feel there should be a more straight-forward solution tham
    #  ...intercepting the init process to determine the schema (or template uri or so) for the style attribute.
    #  ...Hoewever, I did not manage to simply pass the value(!) of e.g. the template field as a style attribute
    #  ...of the JSONField via a SerializerMethodField or similar, although I feel that should be possible.
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        import json

        try:
            if isinstance(self.instance, models.Subtask):
                schema = self.instance.specifications_template.schema
                self.fields['specifications_doc'] = serializers.JSONField(
                    style={'template': 'josdejong_jsoneditor_widget.html',
                           'schema': json.dumps(schema)})
        except Exception as e:
            # todo: Shall we use one of the default templates for the init?
            logger.exception('Could not determine schema, hence no fancy JSON form. Expected for list view.')

    class Meta:
        model = models.Subtask
        fields = '__all__'